@@ -21,6 +21,7 @@ This module contains Databricks operators.
 """
 
 import time
+from typing import Union, Optional, Any, Dict, List
 
 from airflow.exceptions import AirflowException
 from airflow.models import BaseOperator
@@ -31,7 +32,7 @@ XCOM_RUN_ID_KEY = 'run_id'
 XCOM_RUN_PAGE_URL_KEY = 'run_page_url'
 
 
-def _deep_string_coerce(content, json_path='json'):
+def _deep_string_coerce(content, json_path: str = 'json') -> Union[str, list, dict]:
     """
     Coerces content or all values of content if it is a dict to a string. The
     function will throw if content contains non-string or non-numeric types.
@@ -43,22 +44,26 @@ def _deep_string_coerce(content, json_path='json'):
     coerce = _deep_string_coerce
     if isinstance(content, str):
         return content
-    elif isinstance(content, (int, float,)):
+    elif isinstance(
+        content,
+        (
+            int,
+            float,
+        ),
+    ):
         # Databricks can tolerate either numeric or string types in the API backend.
         return str(content)
     elif isinstance(content, (list, tuple)):
         return [coerce(e, '{0}[{1}]'.format(json_path, i)) for i, e in enumerate(content)]
     elif isinstance(content, dict):
-        return {k: coerce(v, '{0}[{1}]'.format(json_path, k))
-                for k, v in list(content.items())}
+        return {k: coerce(v, '{0}[{1}]'.format(json_path, k)) for k, v in list(content.items())}
     else:
         param_type = type(content)
-        msg = 'Type {0} used for parameter {1} is not a number or a string' \
-            .format(param_type, json_path)
+        msg = 'Type {0} used for parameter {1} is not a number or a string'.format(param_type, json_path)
         raise AirflowException(msg)
 
 
-def _handle_databricks_operator_execution(operator, hook, log, context):
+def _handle_databricks_operator_execution(operator, hook, log, context) -> None:
     """
     Handles the Airflow + Databricks lifecycle logic for a Databricks operator
 
@@ -81,9 +86,7 @@ def _handle_databricks_operator_execution(operator, hook, log, context):
                 log.info('View run status, Spark UI, and logs at %s', run_page_url)
                 return
             else:
-                error_message = '{t} failed with terminal state: {s}'.format(
-                    t=operator.task_id,
-                    s=run_state)
+                error_message = '{t} failed with terminal state: {s}'.format(t=operator.task_id, s=run_state)
                 raise AirflowException(error_message)
         else:
             log.info('%s in run state: %s', operator.task_id, run_state)
@@ -236,6 +239,7 @@ class DatabricksSubmitRunOperator(BaseOperator):
     :param do_xcom_push: Whether we should push run_id and run_page_url to xcom.
     :type do_xcom_push: bool
     """
+
     # Used in airflow.models.BaseOperator
     template_fields = ('json',)
     # Databricks brand color (blue) under white text
@@ -245,23 +249,25 @@ class DatabricksSubmitRunOperator(BaseOperator):
     # pylint: disable=too-many-arguments
     @apply_defaults
     def __init__(
-            self,
-            json=None,
-            spark_jar_task=None,
-            notebook_task=None,
-            spark_python_task=None,
-            spark_submit_task=None,
-            new_cluster=None,
-            existing_cluster_id=None,
-            libraries=None,
-            run_name=None,
-            timeout_seconds=None,
-            databricks_conn_id='databricks_default',
-            polling_period_seconds=30,
-            databricks_retry_limit=3,
-            databricks_retry_delay=1,
-            do_xcom_push=False,
-            **kwargs):
+        self,
+        *,
+        json: Optional[Any] = None,
+        spark_jar_task: Optional[Dict[str, str]] = None,
+        notebook_task: Optional[Dict[str, str]] = None,
+        spark_python_task: Optional[Dict[str, Union[str, List[str]]]] = None,
+        spark_submit_task: Optional[Dict[str, List[str]]] = None,
+        new_cluster: Optional[Dict[str, object]] = None,
+        existing_cluster_id: Optional[str] = None,
+        libraries: Optional[List[Dict[str, str]]] = None,
+        run_name: Optional[str] = None,
+        timeout_seconds: Optional[int] = None,
+        databricks_conn_id: str = 'databricks_default',
+        polling_period_seconds: int = 30,
+        databricks_retry_limit: int = 3,
+        databricks_retry_delay: int = 1,
+        do_xcom_push: bool = False,
+        **kwargs,
+    ) -> None:
         """
         Creates a new ``DatabricksSubmitRunOperator``.
         """
@@ -297,11 +303,12 @@ class DatabricksSubmitRunOperator(BaseOperator):
         self.run_id = None
         self.do_xcom_push = do_xcom_push
 
-    def _get_hook(self):
+    def _get_hook(self) -> DatabricksHook:
         return DatabricksHook(
             self.databricks_conn_id,
             retry_limit=self.databricks_retry_limit,
-            retry_delay=self.databricks_retry_delay)
+            retry_delay=self.databricks_retry_delay,
+        )
 
     def execute(self, context):
         hook = self._get_hook()
@@ -311,10 +318,7 @@ class DatabricksSubmitRunOperator(BaseOperator):
     def on_kill(self):
         hook = self._get_hook()
         hook.cancel_run(self.run_id)
-        self.log.info(
-            'Task: %s with run_id: %s was requested to be cancelled.',
-            self.task_id, self.run_id
-        )
+        self.log.info('Task: %s with run_id: %s was requested to be cancelled.', self.task_id, self.run_id)
 
 
 class DatabricksRunNowOperator(BaseOperator):
@@ -448,6 +452,7 @@ class DatabricksRunNowOperator(BaseOperator):
     :param do_xcom_push: Whether we should push run_id and run_page_url to xcom.
     :type do_xcom_push: bool
     """
+
     # Used in airflow.models.BaseOperator
     template_fields = ('json',)
     # Databricks brand color (blue) under white text
@@ -457,19 +462,20 @@ class DatabricksRunNowOperator(BaseOperator):
     # pylint: disable=too-many-arguments
     @apply_defaults
     def __init__(
-            self,
-            job_id=None,
-            json=None,
-            notebook_params=None,
-            python_params=None,
-            spark_submit_params=None,
-            databricks_conn_id='databricks_default',
-            polling_period_seconds=30,
-            databricks_retry_limit=3,
-            databricks_retry_delay=1,
-            do_xcom_push=False,
-            **kwargs):
-
+        self,
+        *,
+        job_id: Optional[str] = None,
+        json: Optional[Any] = None,
+        notebook_params: Optional[Dict[str, str]] = None,
+        python_params: Optional[List[str]] = None,
+        spark_submit_params: Optional[List[str]] = None,
+        databricks_conn_id: str = 'databricks_default',
+        polling_period_seconds: int = 30,
+        databricks_retry_limit: int = 3,
+        databricks_retry_delay: int = 1,
+        do_xcom_push: bool = False,
+        **kwargs,
+    ) -> None:
         """
         Creates a new ``DatabricksRunNowOperator``.
         """
@@ -494,11 +500,12 @@ class DatabricksRunNowOperator(BaseOperator):
         self.run_id = None
         self.do_xcom_push = do_xcom_push
 
-    def _get_hook(self):
+    def _get_hook(self) -> DatabricksHook:
         return DatabricksHook(
             self.databricks_conn_id,
             retry_limit=self.databricks_retry_limit,
-            retry_delay=self.databricks_retry_delay)
+            retry_delay=self.databricks_retry_delay,
+        )
 
     def execute(self, context):
         hook = self._get_hook()
@@ -508,7 +515,4 @@ class DatabricksRunNowOperator(BaseOperator):
     def on_kill(self):
         hook = self._get_hook()
         hook.cancel_run(self.run_id)
-        self.log.info(
-            'Task: %s with run_id: %s was requested to be cancelled.',
-            self.task_id, self.run_id
-        )
+        self.log.info('Task: %s with run_id: %s was requested to be cancelled.', self.task_id, self.run_id)
